@@ -12,6 +12,7 @@ import numpy as np
 
 from sklearn.utils import check_array
 
+from ..information import solver_statistics
 from ..logging import Logger
 from .auto_monotonic import auto_monotonic
 from .auto_monotonic import peak_valley_trend_change_heuristic
@@ -19,12 +20,16 @@ from .binning import OptimalBinning
 from .binning_statistics import multiclass_bin_info
 from .binning_statistics import MulticlassBinningTable
 from .binning_statistics import target_info
+from .binning_statistics import target_info_special_multiclass
 from .multiclass_cp import MulticlassBinningCP
 from .multiclass_mip import MulticlassBinningMIP
 from .preprocessing import split_data
 from .transformations import transform_multiclass_target
 
 
+logger = Logger(__name__).logger
+
+
 def _check_parameters(name, prebinning_method, solver, max_n_prebins,
                       min_prebin_size, min_n_bins, max_n_bins, min_bin_size,
                       max_bin_size, monotonic_trend, max_pvalue,
@@ -145,8 +150,13 @@ def _check_parameters(name, prebinning_method, solver, max_n_prebins,
                                                    len(user_splits_fixed)))
 
     if special_codes is not None:
-        if not isinstance(special_codes, (np.ndarray, list)):
-            raise TypeError("special_codes must be a list or numpy.ndarray.")
+        if not isinstance(special_codes, (np.ndarray, list, dict)):
+            raise TypeError("special_codes must be a dit, list or "
+                            "numpy.ndarray.")
+
+        if isinstance(special_codes, dict) and not len(special_codes):
+            raise ValueError("special_codes empty. special_codes dict must "
+                             "contain at least one special.")
 
     if split_digits is not None:
         if (not isinstance(split_digits, numbers.Integral) or
@@ -247,7 +257,7 @@ class MulticlassOptimalBinning(OptimalBinning):
     user_splits_fixed : array-like or None (default=None)
         The list of pre-binning split points that must be fixed.
 
-    special_codes : array-like or None, optional (default=None)
+    special_codes : array-like, dict or None, optional (default=None)
         List of special codes. Use special codes to specify the data values
         that must be treated separately.
 
@@ -345,12 +355,9 @@ class MulticlassOptimalBinning(OptimalBinning):
         self._time_preprocessing = None
         self._time_prebinning = None
         self._time_solver = None
+        self._time_optimizer = None
         self._time_postprocessing = None
 
-        # logger
-        self._class_logger = Logger(__name__)
-        self._logger = self._class_logger.logger
-
         self._is_fitted = False
 
     def fit(self, x, y, check_input=False):
@@ -473,20 +480,20 @@ class MulticlassOptimalBinning(OptimalBinning):
         time_init = time.perf_counter()
 
         if self.verbose:
-            self._logger.info("Optimal binning started.")
-            self._logger.info("Options: check parameters.")
+            logger.info("Optimal binning started.")
+            logger.info("Options: check parameters.")
 
         _check_parameters(**self.get_params())
 
         # Pre-processing
         if self.verbose:
-            self._logger.info("Pre-processing started.")
+            logger.info("Pre-processing started.")
 
         self._n_samples = len(x)
 
         if self.verbose:
-            self._logger.info("Pre-processing: number of samples: {}"
-                              .format(self._n_samples))
+            logger.info("Pre-processing: number of samples: {}"
+                        .format(self._n_samples))
 
         time_preprocessing = time.perf_counter()
 
@@ -508,26 +515,26 @@ class MulticlassOptimalBinning(OptimalBinning):
             n_missing = len(x_missing)
             n_special = len(x_special)
 
-            self._logger.info("Pre-processing: number of clean samples: {}"
-                              .format(n_clean))
+            logger.info("Pre-processing: number of clean samples: {}"
+                        .format(n_clean))
 
-            self._logger.info("Pre-processing: number of missing samples: {}"
-                              .format(n_missing))
+            logger.info("Pre-processing: number of missing samples: {}"
+                        .format(n_missing))
 
-            self._logger.info("Pre-processing: number of special samples: {}"
-                              .format(n_special))
+            logger.info("Pre-processing: number of special samples: {}"
+                        .format(n_special))
 
             if self.outlier_detector is not None:
                 n_outlier = self._n_samples-(n_clean + n_missing + n_special)
-                self._logger.info("Pre-processing: number of outlier samples: "
-                                  "{}".format(n_outlier))
+                logger.info("Pre-processing: number of outlier samples: {}"
+                            .format(n_outlier))
 
-            self._logger.info("Pre-processing terminated. Time: {:.4f}s"
-                              .format(self._time_preprocessing))
+            logger.info("Pre-processing terminated. Time: {:.4f}s"
+                        .format(self._time_preprocessing))
 
         # Pre-binning
         if self.verbose:
-            self._logger.info("Pre-binning started.")
+            logger.info("Pre-binning started.")
 
         time_prebinning = time.perf_counter()
 
@@ -535,8 +542,8 @@ class MulticlassOptimalBinning(OptimalBinning):
             n_splits = len(self.user_splits)
 
             if self.verbose:
-                self._logger.info("Pre-binning: user splits supplied: {}"
-                                  .format(n_splits))
+                logger.info("Pre-binning: user splits supplied: {}"
+                            .format(n_splits))
 
             user_splits = check_array(self.user_splits, ensure_2d=False,
                                       dtype=None, force_all_finite=True)
@@ -552,31 +559,32 @@ class MulticlassOptimalBinning(OptimalBinning):
                     self.user_splits_fixed)[sorted_idx]
 
             splits, n_nonevent, n_event = self._prebinning_refinement(
-                user_splits, x_clean, y_clean, y_missing, y_special, None)
+                user_splits, x_clean, y_clean, y_missing, x_special, y_special,
+                None)
         else:
             splits, n_nonevent, n_event = self._fit_prebinning(
-                x_clean, y_clean, y_missing, y_special, None)
+                x_clean, y_clean, y_missing, x_special, y_special, None)
 
         self._n_prebins = len(n_nonevent)
 
         self._time_prebinning = time.perf_counter() - time_prebinning
 
         if self.verbose:
-            self._logger.info("Pre-binning: number of prebins: {}"
-                              .format(self._n_prebins))
-            self._logger.info("Pre-binning: number of refinements: {}"
-                              .format(self._n_refinements))
+            logger.info("Pre-binning: number of prebins: {}"
+                        .format(self._n_prebins))
+            logger.info("Pre-binning: number of refinements: {}"
+                        .format(self._n_refinements))
 
-            self._logger.info("Pre-binning terminated. Time: {:.4f}s"
-                              .format(self._time_prebinning))
+            logger.info("Pre-binning terminated. Time: {:.4f}s"
+                        .format(self._time_prebinning))
 
         # Optimization
         self._fit_optimizer(splits, n_nonevent, n_event)
 
         # Post-processing
         if self.verbose:
-            self._logger.info("Post-processing started.")
-            self._logger.info("Post-processing: compute binning information.")
+            logger.info("Post-processing started.")
+            logger.info("Post-processing: compute binning information.")
 
         time_postprocessing = time.perf_counter()
 
@@ -591,30 +599,29 @@ class MulticlassOptimalBinning(OptimalBinning):
             self._n_event_special)
 
         self._binning_table = MulticlassBinningTable(
-            self.name, self._splits_optimal, self._n_event, self._classes)
+            self.name, self.special_codes, self._splits_optimal, self._n_event,
+            self._classes)
 
         self._time_postprocessing = time.perf_counter() - time_postprocessing
 
         if self.verbose:
-            self._logger.info("Post-processing terminated. Time: {:.4f}s"
-                              .format(self._time_postprocessing))
+            logger.info("Post-processing terminated. Time: {:.4f}s"
+                        .format(self._time_postprocessing))
 
         self._time_total = time.perf_counter() - time_init
 
         if self.verbose:
-            self._logger.info("Optimal binning terminated. Status: {}. "
-                              "Time: {:.4f}s"
-                              .format(self._status, self._time_total))
+            logger.info("Optimal binning terminated. Status: {}. Time: {:.4f}s"
+                        .format(self._status, self._time_total))
 
         # Completed successfully
-        self._class_logger.close()
         self._is_fitted = True
 
         return self
 
     def _prebinning_refinement(self, splits_prebinning, x, y, y_missing,
-                               y_special, y_others=None, sw_clean=None,
-                               sw_missing=None, sw_special=None,
+                               x_special, y_special, y_others=None,
+                               sw_clean=None, sw_missing=None, sw_special=None,
                                sw_others=None):
 
         self._classes = np.unique(y)
@@ -624,14 +631,10 @@ class MulticlassOptimalBinning(OptimalBinning):
             raise ValueError("Maximum number of classes exceeded; got {}."
                              .format(self._n_classes))
 
-        special_target_info = []
-        missing_target_info = []
-        for idx, cl in enumerate(self._classes):
-            special_target_info.append(target_info(y_special, cl)[0])
-            missing_target_info.append(target_info(y_missing, cl)[0])
-
-        self._n_event_special = special_target_info
-        self._n_event_missing = missing_target_info
+        self._n_event_special = target_info_special_multiclass(
+            self.special_codes, x_special, y_special, self._classes)
+        self._n_event_missing = [target_info(y_missing, cl)[0]
+                                 for cl in self._classes]
 
         n_splits = len(splits_prebinning)
 
@@ -648,7 +651,7 @@ class MulticlassOptimalBinning(OptimalBinning):
 
     def _fit_optimizer(self, splits, n_nonevent, n_event):
         if self.verbose:
-            self._logger.info("Optimizer started.")
+            logger.info("Optimizer started.")
 
         time_init = time.perf_counter()
 
@@ -658,10 +661,10 @@ class MulticlassOptimalBinning(OptimalBinning):
             self._solution = np.zeros(len(splits)).astype(bool)
 
             if self.verbose:
-                self._logger.warning("Optimizer: no bins after pre-binning.")
-                self._logger.warning("Optimizer: solver not run.")
+                logger.warning("Optimizer: no bins after pre-binning.")
+                logger.warning("Optimizer: solver not run.")
 
-                self._logger.info("Optimizer terminated. Time: 0s")
+                logger.info("Optimizer terminated. Time: 0s")
             return
 
         if self.min_bin_size is not None:
@@ -684,8 +687,8 @@ class MulticlassOptimalBinning(OptimalBinning):
                          for i in range(len(self._classes))]
 
             if self.verbose:
-                self._logger.info("Optimizer: classifier predicts {} "
-                                  "monotonic trends.".format(monotonic))
+                logger.info("Optimizer: classifier predicts {} "
+                            "monotonic trends.".format(monotonic))
         elif isinstance(self.monotonic_trend, list):
             if len(self.monotonic_trend) != self._n_classes:
                 raise ValueError("List of monotonic trends must be of size "
@@ -713,15 +716,15 @@ class MulticlassOptimalBinning(OptimalBinning):
                     monotonic.append(trend)
 
                     if self.verbose:
-                        self._logger.info("Optimizer: classifier predicts {} "
-                                          "monotonic trend.".format(trend))
+                        logger.info("Optimizer: classifier predicts {} "
+                                    "monotonic trend.".format(trend))
                 else:
                     monotonic.append(m_trend)
         elif self.monotonic_trend is None:
             monotonic = [None] * self._n_classes
 
             if self.verbose:
-                self._logger.info("Optimizer: monotonic trend not set.")
+                logger.info("Optimizer: monotonic trend not set.")
 
         if self.solver == "cp":
             optimizer = MulticlassBinningCP(monotonic, self.min_n_bins,
@@ -739,26 +742,27 @@ class MulticlassOptimalBinning(OptimalBinning):
                                              self.user_splits_fixed,
                                              self.time_limit)
         if self.verbose:
-            self._logger.info("Optimizer: build model...")
+            logger.info("Optimizer: build model...")
 
         optimizer.build_model(n_nonevent, n_event, trend_changes)
 
         if self.verbose:
-            self._logger.info("Optimizer: solve...")
+            logger.info("Optimizer: solve...")
 
         status, solution = optimizer.solve()
 
         self._solution = solution
 
-        self._optimizer = optimizer
+        self._optimizer, self._time_optimizer = solver_statistics(
+            self.solver, optimizer.solver_)
         self._status = status
         self._splits_optimal = splits[solution[:-1]]
 
         self._time_solver = time.perf_counter() - time_init
 
         if self.verbose:
-            self._logger.info("Optimizer terminated. Time: {:.4f}s"
-                              .format(self._time_solver))
+            logger.info("Optimizer terminated. Time: {:.4f}s"
+                        .format(self._time_solver))
 
     def _compute_prebins(self, splits_prebinning, x, y):
         n_splits = len(splits_prebinning)
@@ -808,8 +812,8 @@ class MulticlassOptimalBinning(OptimalBinning):
             splits = splits_prebinning[~mask_splits]
 
             if self.verbose:
-                self._logger.info("Pre-binning: number prebins removed: {}"
-                                  .format(np.count_nonzero(mask_remove)))
+                logger.info("Pre-binning: number prebins removed: {}"
+                            .format(np.count_nonzero(mask_remove)))
 
             [splits_prebinning, n_nonevent, n_event] = self._compute_prebins(
                 splits, x, y)
